/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include "DynamicArray.h"

/*
 * This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

struct DynamicArray {
  size_t max_size;
  size_t len;
  FreeFunc free_func;
  _pointer * array;
};

void _private_for_each_item_free (DynamicArray * self, _pointer item,
                                    _pointer data);

DynamicArray * dynamic_array_new (size_t len, FreeFunc free_func) {
  DynamicArray * self = malloc (sizeof(DynamicArray));
  
  self->max_size = len;
  self->len = 0;
  
  self->free_func = free_func;
  
  self->array = malloc (len * sizeof (* self->array));
  
  return self;
}

void dynamic_array_free (DynamicArray * self) {
  if (self->free_func != NULL) {
    dynamic_array_for_each (self, (ForEachFunc) _private_for_each_item_free,
                            NULL);
  } else {
    for (int i = 0; i < self->len; i++) {
      free (self->array[i]);
    }
  }
  free (self->array);
  free (self);
}

_pointer dynamic_array_get (DynamicArray * self, size_t index) {
  return self->array[index];
}

size_t dynamic_array_len (DynamicArray * self) {
  return self->len;
}

size_t dynamic_array_size (DynamicArray * self) {
  return self->max_size;
}

void dynamic_array_add (DynamicArray * self, _pointer data) {
  if (self->len >= self->max_size - 1) {
    self->array = realloc (self->array,
                          (sizeof (* self->array) *
                          (self->max_size + ARRAY_PADDING)));
    self->max_size = self->max_size + ARRAY_PADDING;
  }
  self->array[self->len] = data;
  self->len++;
}

_pointer * dynamic_array_dump_array (DynamicArray * self) {
  _pointer * ret_val = malloc (sizeof (* self->array) * (self->len + 1));
  for (int i = 0; i < self->len; i++) {
    ret_val[i] = self->array[i];
  }
  ret_val[self->len] = NULL;
  return ret_val;
}

void dynamic_array_for_each (DynamicArray * self, ForEachFunc func,
                             _pointer data) {
  for (int i = 0; i < self->len; i++) {
    func (self, self->array[i], data);
  }
}


void _private_for_each_item_free (DynamicArray * self, _pointer item,
                                    _pointer data) {
  FreeFunc func = self->free_func;
  func (item);
}

