/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#ifndef __H_GAME__
#define __H_GAME__


#include "defs.h"
#include <SDL2/SDL.h>
#include <stdbool.h>

BEGIN_DECLS

#include "MousePointer.h"
#include "GameButton.h"
//#include "JSParser.h"
#include "game_utils.h"

/* This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

#define J_SCALE_FACTOR 5000 //TODO: change the movement to time-based

/** @file
 * A Game represents the game itself. This is going to be re-implementation in
 * the future.
 */

/**
 * Quits the program in an unsafe way.
 */
void quit (int rc);

typedef struct Game {
  SDL_Event event;
  SDL_Point mouse_delta;
  bool mouse_pressed;
  bool bg_color_toggle;
  int done;
  SDL_Color background_color;
  SDL_Window * window;
  SDL_Renderer * renderer;
  SDL_GameController * game_controller;
  MousePointer * mouse;
  GameButton * button;
} Game;

/**
 * Creates a new Game.
 */
Game * game_new ();

/**
 * The function that possesses the events of the game.
 * Should generally not be called from outside the Game.
 */
void game_process_events (Game * self);

/**
 * The function that has to be run for the game to run. executes the main loop
 * of the game.
 */
void game_main_loop (Game * self);
/**
 * Frees the Game and the resources associated with it.
 */
void game_free (Game * self);

/**
 * a function that internally is called to draw all the objects on the screen.
 */
void game_draw (Game * self);

END_DECLS

#endif
