/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#ifndef __H_GAME_BUTTON__
#define __H_GAME_BUTTON__

/* This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

#include <SDL2/SDL.h>
#include <stdbool.h>

#include "MousePointer.h"

typedef struct GameButton GameButton; // hack to make the callback work...

typedef void (* BtnCallback)(GameButton * self, void * data);

typedef struct GameButton {
  SDL_Rect base; //this will behave like an SDL_Rect when cast to an SDL_Rect.
  bool fill;
  bool is_clicked;
  bool is_hover;
  bool is_stop_hover;
  
  SDL_Color color;
  
  BtnCallback clicked;
  void * clicked_data;
  
  BtnCallback hover;
  void * hover_data;
  
  BtnCallback stop_hover;
  void * stop_hover_data;
} GameButton;

typedef void (* BtnCallback)(GameButton * self, void * data);

GameButton * game_button_new (int x, int y, int h, int w);

bool game_button_check_clicked (GameButton * self, MousePointer * pointer);

bool game_button_check_hover (GameButton * self, MousePointer * pointer);

void game_button_set_callback (GameButton * self,
                               const char * name,
                               BtnCallback callback,
                               void * data);

void game_button_do_callback (GameButton * self,
                              const char * name);

void game_button_draw (GameButton * self, SDL_Renderer * renderer);

void game_button_set_fill (GameButton * self, bool fill);

void game_button_free (GameButton * self);
#endif
