/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "game_utils.h"
#include "JSParser.h"

/*
 * This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

/* Js/C function declarations. */
int js_print_error (duk_context * ctx);


/******************************************************************************/
GameJSParser * game_js_parser_new (Game * game,
                                   char * file_name,
                                   char * script) {
  GameJSParser * self = malloc (sizeof(GameJSParser));
  if (file_name != NULL && script != NULL) {
    print_error (
      "You can not use both a script and a file when creating a GameJSParser.\n"
      "Exacly ONE of these must be used."
    );
    goto error;
  } else if (file_name == NULL && script == NULL) {
    print_error (
      "WARNING: No script or file is set when constructing GameJSParser"
    );
  }
  
  /* Functions to be added to the environment,
   * realised at the end of this file.
   */
  const duk_function_list_entry js_funcs[] = {
    {"print_error", js_print_error, 1},
    {NULL,NULL,0}
  };
  duk_put_function_list (self->ctx, -1, js_funcs);
  
  /* Set some stuffs and evaluates the scripts */
  if (file_name) {
    self->file_name = file_name;
    duk_eval_file (self->ctx, self->file_name);
    duk_pop (self->ctx);
  }
  if (script) {
    self->script = script;
    duk_eval_string (self->ctx, self->script);
    duk_pop (self->ctx);
  }
  
  self->ctx = duk_create_heap_default ();
  if (!self->ctx) {
    print_error ("Could not Initialise Duktape Context.");
    goto error;
  }
  
  self->game = game;
  
  return self;
  
  error:
  game_js_parser_free (self);
  return NULL;
}

void game_js_parser_free (GameJSParser * self) {
  duk_destroy_heap(self->ctx);
  if (strlen (self->file_name)) {
    free (self->file_name);
  }
  if (strlen (self->script)) {
    free (self->script);
  }
  
  free (self);
}

void game_js_parser_set_settings_loader (GameJSParser * self,
                                              GameJSParserLoadDataFunc func) {
  self->load_data_func = func;
}

void game_js_parser_set_settings_dump (GameJSParser * self,
                                              GameJSParserDumpDataFunc func) {
  self->dump_data_func = func;
}

void game_js_parser_load_settings (GameJSParser * self, void * data) {
  GameJSParserLoadDataFunc func = self->load_data_func;
  func (self, data);
}
void * game_js_parser_dump_settings (GameJSParser * self) {
  GameJSParserDumpDataFunc func = self->dump_data_func;
  return func (self);
}

void game_js_parser_run (GameJSParser * self) {
  
  
}

/******************************************************************************/
int js_print_error (duk_context * ctx) {
  /* function print_error (str) */
  
  const char * buf = duk_get_string (ctx, 0);
  if (buf) {
    print_error (buf);
    return 0;
  }
  return -1;
}

