/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

#include "MousePointer.h"
#include "game_utils.h"

/*
 * This file, as the rest of the project is under MIT license.
 * see http://opensource.org/licenses/MIT
 *
 * Author Gustav Hartvigsson <gustav.hartvigsson _at_ gmail.com> 2014
 */

MousePointer * mouse_pointer_new (int  b_box_h, int b_box_w, SDL_Color * color){
  MousePointer * self = malloc (sizeof(MousePointer));
  fprintf (stdout, "Creating MousePointer: address: %ld\n", (long) self);
  /* TODO: add image... */
  self->image = NULL;
  self->draw_rect = game_rect_new (0,0,32,32);
  self->bounding_box = game_rect_new (0,0,b_box_h, b_box_w);
  self->possision.x = 0;
  self->possision.y = 0;
  self->pressed = false;
  return self;
}

void mouse_pointer_move (MousePointer * self, int deltax, int deltay) {
  self->draw_rect->x += deltax;
  self->draw_rect->y += deltay;
  self->possision.x += deltax;
  self->possision.y += deltay;
}

void mouse_pointer_move_to (MousePointer * self, int x, int y) {
  self->draw_rect->x = x;
  self->draw_rect->y = y;
  self->possision.x = x; /* Will change to some x and y plus off set */
  self->possision.y = y;
}


void mouse_pointer_draw (MousePointer * self, SDL_Renderer * renderer) {
  int ret = SDL_SetRenderDrawColor(renderer,255,0,0,255);
  if (ret < 0) {
    fprintf(stderr, "Couldn't set Colour: %s\n", SDL_GetError());
  }
  
  ret = SDL_RenderDrawRect (renderer, self->draw_rect);
  if (ret < 0) {
    fprintf(stderr, "Couldn't render MousePointer: %s\n", SDL_GetError());
  }
  
  mouse_pointer_force_pos (self);
}

void mouse_pointer_force_pos (MousePointer * self) {
  if (self->draw_rect->x > self->bounding_box->h) {
    self->draw_rect->x -= 10;
  }
  if (self->draw_rect->y > self->bounding_box->w) {
    self->draw_rect->y -= 10;
  }
  if (self->draw_rect->y < self->bounding_box->y) {
    self->draw_rect->y += 10;
  }
  if (self->draw_rect->x < self->bounding_box->x) {
    self->draw_rect->x += 10;
  }
  self->possision.x = self->draw_rect->x; /* Again, some ofset can be needed. */
  self->possision.y = self->draw_rect->y;
}

void mouse_pointer_free (MousePointer * self) {
  fprintf (stdout, "Freeing MousePointer: address: %ld\n", (long) self);
  if (self->image) {
    SDL_DestroyTexture (self->image);
  }
  free (self->draw_rect);
  free (self->bounding_box);
  free (self);
}
